
package tiny;

%%

%class Scanner
%function getToken
%type Token
%yylexthrow Exception
%line
%column

%state STRING

%{
  StringBuilder sb;
%}

digit = [0-9]
letter = [a-zA-Z]
alpha = {letter} | {digit} | [_]
space = [ \n\t]
comment = "{"~"}"

%%

<YYINITIAL> {

"if"    { return new Token(Token.IF); }
"then"    { return new Token(Token.THEN); }
"else"    { return new Token(Token.ELSE); }
"end"    { return new Token(Token.END); }
"repeat"    { return new Token(Token.REPEAT); }
"until"    { return new Token(Token.UNTIL); }
"read"    { return new Token(Token.READ); }
"write"    { return new Token(Token.WRITE); }
"var"    { return new Token(Token.VAR); }
"and"    { return new Token(Token.AND); }
"or"     { return new Token(Token.OR); }
"not"     { return new Token(Token.NOT); }
"function" { return new Token(Token.FUNCTION); }
"return" { return new Token(Token.RETURN); }
"break"  { return new Token(Token.BREAK); }

"+" | "-" | "*" | "/" | "=" | "<" | "(" | ")" | ";" | "," | "[" | "]"  { return new Token((int)yytext().charAt(0)); }

":="   { return new Token(Token.ATTRIB); }

{letter}{alpha}*	     { return new Token(Token.ID, yytext()); }

{digit}+   { return new Token(Token.NUM, Integer.parseInt(yytext())); }

{space}    { }

{comment}    { }

[']     { yybegin(STRING); sb = new StringBuilder(); }

<<EOF>>   { return new Token(Token.EOF); }

}

<STRING> {

['][']  { sb.append("'"); }

#{digit}{digit}{digit}  { 
  String scode = yytext().substring(1, 4);
  int icode = Integer.parseInt(scode);
  sb.append(Character.toString((char)icode));
}

##    { sb.append("#"); }

[^'#\n]  { sb.append(yytext()); }

[']   { yybegin(YYINITIAL); return new Token(Token.STRING, sb.toString()); }

}

.  { throw new Exception("scanner error at line " + yyline + " column " + yycolumn + ", token: " + yytext()); }
